/*
 * Renders the image texture with its assocated mask, tint color, and rounded corners.
 */

precision mediump float;

uniform sampler2D imageTexture;
uniform sampler2D maskTexture;
uniform sampler2D maskOverlayTexture;
uniform vec2 textureScale;
uniform vec2 textureOffset;
uniform vec4 tintColor;
uniform vec4 maskColor;
uniform float backgroundAlpha;
uniform vec2 cornerEllipse;

varying vec2 rectCoordinate;

bool isPointInEllipse(vec2 point) {
    float a = (point.x * point.x) / (cornerEllipse.x * cornerEllipse.x) +
    (point.y * point.y) / (cornerEllipse.y * cornerEllipse.y);

    return a < 1.0;
}

void discardRoundedCorners(vec2 p) {
    if (p.x < cornerEllipse.x && p.y < cornerEllipse.y && !isPointInEllipse(p - cornerEllipse)) {
        discard;
    }

    if (p.x < cornerEllipse.x && p.y > (1.0 - cornerEllipse.y) &&
        !isPointInEllipse(p - vec2(cornerEllipse.x, 1.0 - cornerEllipse.y))) {
        discard;
    }

    if (p.x > (1.0 - cornerEllipse.x) && p.y < cornerEllipse.y &&
        !isPointInEllipse(p - vec2(1.0 - cornerEllipse.x, cornerEllipse.y))) {
        discard;
    }

    if (p.x > (1.0 - cornerEllipse.x) && p.y > (1.0 - cornerEllipse.y) &&
        !isPointInEllipse(p - vec2(1.0 - cornerEllipse.x, 1.0 - cornerEllipse.y))) {
        discard;
    }
}

void main() {
    discardRoundedCorners(rectCoordinate);

    // Texture coordinate
    vec2 textureCoordinate = rectCoordinate * textureScale + textureOffset;

    // Discard samples that lie outside the texture
    if (textureCoordinate.x < 0.0 || textureCoordinate.y < 0.0 ||
        textureCoordinate.x > 1.0 || textureCoordinate.y > 1.0) {
        discard;
    }

    // Sample the main image and set its alpha to the value in the mask image
    vec4 result = vec4(texture2D(imageTexture, textureCoordinate).rgb * tintColor.rgb,
                       max(backgroundAlpha, texture2D(maskTexture, textureCoordinate).r));

    // If the mask overlay is set then apply the mask color to this pixel
    float maskOverlayAlpha = texture2D(maskOverlayTexture, textureCoordinate).r * maskColor.a;
    result.rgb = mix(result.rgb, maskColor.rgb, maskOverlayAlpha) * result.a;
    gl_FragColor = result;
}
